@@ -5,6 +5,7 @@ import numpy as np
 from numpy.testing import (assert_array_almost_equal, assert_almost_equal,
                            assert_array_equal, assert_allclose)
 from nose.tools import assert_equal, assert_true, assert_raises
+import pytest
 from scipy.signal import resample as sp_resample, butter
 from scipy.fftpack import fft, fftfreq
 
@@ -15,7 +16,7 @@ from mne.filter import (filter_data, resample, _resample_stim_channels,
                         _overlap_add_filter, _smart_pad, design_mne_c_filter,
                         estimate_ringing_samples, create_filter, _Interp2)
 
-from mne.utils import (sum_squared, run_tests_if_main, slow_test,
+from mne.utils import (sum_squared, run_tests_if_main,
                        catch_logging, requires_version, _TempDir,
                        requires_mne, run_subprocess)
 
@@ -85,7 +86,7 @@ def test_1d_filter():
                     h = np.concatenate([[1.], np.zeros(n_filter - 1)])
                 # ensure we pad the signal the same way for both filters
                 n_pad = n_filter - 1
-                x_pad = _smart_pad(x, np.array([n_pad, n_pad]))
+                x_pad = _smart_pad(x, (n_pad, n_pad))
                 for phase in ('zero', 'linear', 'zero-double'):
                     # compute our expected result the slow way
                     if phase == 'zero':
@@ -219,7 +220,8 @@ def test_notch_filters():
     for meth, lf, fl, tol in zip(methods, line_freqs, filter_lengths, tols):
         with catch_logging() as log_file:
             with warnings.catch_warnings(record=True):
-                b = notch_filter(a, sfreq, lf, fl, method=meth, verbose=True)
+                b = notch_filter(a, sfreq, lf, fl, method=meth,
+                                 fir_design='firwin', verbose=True)
         if lf is None:
             out = log_file.getvalue().split('\n')[:-1]
             if len(out) != 2 and len(out) != 3:  # force_serial: len(out) == 3
@@ -278,7 +280,7 @@ def test_resample_stim_channel():
 
 
 @requires_version('scipy', '0.16')
-@slow_test
+@pytest.mark.slowtest
 def test_filters():
     """Test low-, band-, high-pass, and band-stop filters plus resampling."""
     sfreq = 100
@@ -289,41 +291,46 @@ def test_filters():
     # let's test our catchers
     for fl in ['blah', [0, 1], 1000.5, '10ss', '10']:
         assert_raises(ValueError, filter_data, a, sfreq, 4, 8, None, fl,
-                      1.0, 1.0)
+                      1.0, 1.0, fir_design='firwin')
     for nj in ['blah', 0.5]:
         assert_raises(ValueError, filter_data, a, sfreq, 4, 8, None, 1000,
-                      1.0, 1.0, n_jobs=nj, phase='zero', fir_window='hann')
+                      1.0, 1.0, n_jobs=nj, phase='zero', fir_design='firwin')
     assert_raises(ValueError, filter_data, a, sfreq, 4, 8, None, 100,
                   1., 1., fir_window='foo')
+    assert_raises(ValueError, filter_data, a, sfreq, 4, 8, None, 10,
+                  1., 1., fir_design='firwin')  # too short
     # > Nyq/2
     assert_raises(ValueError, filter_data, a, sfreq, 4, sfreq / 2., None,
-                  100, 1.0, 1.0)
+                  100, 1.0, 1.0, fir_design='firwin')
     assert_raises(ValueError, filter_data, a, sfreq, -1, None, None,
-                  100, 1.0, 1.0)
+                  100, 1.0, 1.0, fir_design='firwin')
     # these should work
-    create_filter(a, sfreq, None, None)
+    create_filter(a, sfreq, None, None, fir_design='firwin')
     create_filter(a, sfreq, None, None, method='iir')
 
     # check our short-filter warning:
     with warnings.catch_warnings(record=True) as w:
         # Warning for low attenuation
-        filter_data(a, sfreq, 1, 8, filter_length=256)
+        filter_data(a, sfreq, 1, 8, filter_length=256, fir_design='firwin2')
     assert_true(any('attenuation' in str(ww.message) for ww in w))
     with warnings.catch_warnings(record=True) as w:
         # Warning for too short a filter
-        filter_data(a, sfreq, 1, 8, filter_length='0.5s')
+        filter_data(a, sfreq, 1, 8, filter_length='0.5s', fir_design='firwin2')
     assert_true(any('Increase filter_length' in str(ww.message) for ww in w))
 
     # try new default and old default
     freqs = fftfreq(a.shape[-1], 1. / sfreq)
     A = np.abs(fft(a))
+    kwargs = dict(fir_design='firwin')
     for fl in ['auto', '10s', '5000ms', 1024, 1023]:
-        bp = filter_data(a, sfreq, 4, 8, None, fl, 1.0, 1.0)
-        bs = filter_data(a, sfreq, 8 + 1.0, 4 - 1.0, None, fl, 1.0, 1.0)
-        lp = filter_data(a, sfreq, None, 8, None, fl, 10, 1.0, n_jobs=2)
-        hp = filter_data(lp, sfreq, 4, None, None, fl, 1.0, 10)
-        assert_array_almost_equal(hp, bp, 4)
-        assert_array_almost_equal(bp + bs, a, 4)
+        bp = filter_data(a, sfreq, 4, 8, None, fl, 1.0, 1.0, **kwargs)
+        bs = filter_data(a, sfreq, 8 + 1.0, 4 - 1.0, None, fl, 1.0, 1.0,
+                         **kwargs)
+        lp = filter_data(a, sfreq, None, 8, None, fl, 10, 1.0, n_jobs=2,
+                         **kwargs)
+        hp = filter_data(lp, sfreq, 4, None, None, fl, 1.0, 10, **kwargs)
+        assert_allclose(hp, bp, rtol=1e-3, atol=1e-3)
+        assert_allclose(bp + bs, a, rtol=1e-3, atol=1e-3)
         # Sanity check ttenuation
         mask = (freqs > 5.5) & (freqs < 6.5)
         assert_allclose(np.mean(np.abs(fft(bp)[:, mask]) / A[:, mask]),
@@ -332,9 +339,9 @@ def test_filters():
                         0., atol=0.2)
         # now the minimum-phase versions
         bp = filter_data(a, sfreq, 4, 8, None, fl, 1.0, 1.0,
-                         phase='minimum')
+                         phase='minimum', **kwargs)
         bs = filter_data(a, sfreq, 8 + 1.0, 4 - 1.0, None, fl, 1.0, 1.0,
-                         phase='minimum')
+                         phase='minimum', **kwargs)
         assert_allclose(np.mean(np.abs(fft(bp)[:, mask]) / A[:, mask]),
                         1., atol=0.11)
         assert_allclose(np.mean(np.abs(fft(bs)[:, mask]) / A[:, mask]),
@@ -387,8 +394,10 @@ def test_filters():
     a = rng.randn(5 * sfreq, 5 * sfreq)
     b = a[:, None, :]
 
-    a_filt = filter_data(a, sfreq, 4, 8, None, 400, 2.0, 2.0)
-    b_filt = filter_data(b, sfreq, 4, 8, [0], 400, 2.0, 2.0)
+    a_filt = filter_data(a, sfreq, 4, 8, None, 400, 2.0, 2.0,
+                         fir_design='firwin')
+    b_filt = filter_data(b, sfreq, 4, 8, [0], 400, 2.0, 2.0,
+                         fir_design='firwin')
 
     assert_array_equal(a_filt[:, None, :], b_filt)
 
@@ -411,22 +420,30 @@ def test_filter_auto():
     t = np.arange(N) / sfreq
     x += np.sin(2 * np.pi * sine_freq * t)
     x_orig = x.copy()
-    x_filt = filter_data(x, sfreq, None, lp)
-    assert_array_equal(x, x_orig)
-    # the firwin2 function gets us this close
-    assert_allclose(x, x_filt, rtol=1e-4, atol=1e-4)
-    assert_array_equal(x_filt, filter_data(x, sfreq, None, lp, None))
-    assert_array_equal(x, x_orig)
-    assert_array_equal(x_filt, filter_data(x, sfreq, None, lp))
-    assert_array_equal(x, x_orig)
-    assert_array_equal(x_filt, filter_data(x, sfreq, None, lp, copy=False))
-    assert_array_equal(x, x_filt)
+    for pad in ('reflect_limited', 'reflect', 'edge'):
+        for fir_design in ('firwin2', 'firwin'):
+            kwargs = dict(fir_design=fir_design, pad=pad)
+            x = x_orig.copy()
+            x_filt = filter_data(x, sfreq, None, lp, **kwargs)
+            assert_array_equal(x, x_orig)
+            n_edge = 10
+            assert_allclose(x[n_edge:-n_edge], x_filt[n_edge:-n_edge],
+                            atol=1e-2)
+            assert_array_equal(x_filt, filter_data(x, sfreq, None, lp, None,
+                                                   **kwargs))
+            assert_array_equal(x, x_orig)
+            assert_array_equal(x_filt, filter_data(x, sfreq, None, lp,
+                                                   **kwargs))
+            assert_array_equal(x, x_orig)
+            assert_array_equal(x_filt, filter_data(x, sfreq, None, lp,
+                                                   copy=False, **kwargs))
+            assert_array_equal(x, x_filt)
 
     # degenerate conditions
     assert_raises(ValueError, filter_data, x, -sfreq, 1, 10)
     assert_raises(ValueError, filter_data, x, sfreq, 1, sfreq * 0.75)
     assert_raises(TypeError, filter_data, x.astype(np.float32), sfreq, None,
-                  10, filter_length='auto', h_trans_bandwidth='auto')
+                  10, filter_length='auto', h_trans_bandwidth='auto', **kwargs)
 
 
 def test_cuda():
@@ -438,27 +455,28 @@ def test_cuda():
     sfreq = 500
     sig_len_secs = 20
     a = rng.randn(sig_len_secs * sfreq)
+    kwargs = dict(fir_design='firwin')
 
     with catch_logging() as log_file:
         for fl in ['auto', '10s', 2048]:
             args = [a, sfreq, 4, 8, None, fl, 1.0, 1.0]
-            bp = filter_data(*args)
-            bp_c = filter_data(*args, n_jobs='cuda', verbose='info')
+            bp = filter_data(*args, **kwargs)
+            bp_c = filter_data(*args, n_jobs='cuda', verbose='info', **kwargs)
             assert_array_almost_equal(bp, bp_c, 12)
 
             args = [a, sfreq, 8 + 1.0, 4 - 1.0, None, fl, 1.0, 1.0]
-            bs = filter_data(*args)
-            bs_c = filter_data(*args, n_jobs='cuda', verbose='info')
+            bs = filter_data(*args, **kwargs)
+            bs_c = filter_data(*args, n_jobs='cuda', verbose='info', **kwargs)
             assert_array_almost_equal(bs, bs_c, 12)
 
             args = [a, sfreq, None, 8, None, fl, 1.0]
-            lp = filter_data(*args)
-            lp_c = filter_data(*args, n_jobs='cuda', verbose='info')
+            lp = filter_data(*args, **kwargs)
+            lp_c = filter_data(*args, n_jobs='cuda', verbose='info', **kwargs)
             assert_array_almost_equal(lp, lp_c, 12)
 
             args = [lp, sfreq, 4, None, None, fl, 1.0]
-            hp = filter_data(*args)
-            hp_c = filter_data(*args, n_jobs='cuda', verbose='info')
+            hp = filter_data(*args, **kwargs)
+            hp_c = filter_data(*args, n_jobs='cuda', verbose='info', **kwargs)
             assert_array_almost_equal(hp, hp_c, 12)
 
     # check to make sure we actually used CUDA
