@@ -17,7 +17,7 @@ from ..io.pick import (pick_types, _pick_data_channels, pick_info,
                        _PICK_TYPES_KEYS, pick_channels, channel_type)
 from ..io.proj import setup_proj
 from ..io.meas_info import create_info
-from ..utils import verbose, get_config
+from ..utils import verbose, get_config, _ensure_int
 from ..time_frequency import psd_welch
 from ..defaults import _handle_default
 from .topo import _plot_topo, _plot_timeseries, _plot_timeseries_unified
@@ -27,7 +27,8 @@ from .utils import (_toggle_options, _toggle_proj, tight_layout,
                     _helper_raw_resize, _select_bads, _onclick_help,
                     _setup_browser_offsets, _compute_scalings, plot_sensors,
                     _radio_clicked, _set_radio_button, _handle_topomap_bads,
-                    _change_channel_group, _plot_annotations)
+                    _change_channel_group, _plot_annotations, _setup_butterfly,
+                    _handle_decim)
 from .evoked import _plot_lines
 
 
@@ -89,10 +90,11 @@ def _pick_bad_channels(event, params):
 
 def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
              bgcolor='w', color=None, bad_color=(0.8, 0.8, 0.8),
-             event_color='cyan', scalings=None, remove_dc=True, order='type',
+             event_color='cyan', scalings=None, remove_dc=True, order=None,
              show_options=False, title=None, show=True, block=False,
              highpass=None, lowpass=None, filtorder=4, clipping=None,
-             show_first_samp=False):
+             show_first_samp=False, proj=True, group_by='type',
+             butterfly=False, decim='auto'):
     """Plot raw data.
 
     Parameters
@@ -110,7 +112,7 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
         ``raw.first_samp``.
     n_channels : int
         Number of channels to plot at once. Defaults to 20. Has no effect if
-        ``order`` is 'position' or 'selection'.
+        ``order`` is 'position', 'selection' or 'butterfly'.
     bgcolor : color object
         Color of the background.
     color : dict | color object | None
@@ -139,15 +141,12 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
 
     remove_dc : bool
         If True remove DC component when plotting data.
-    order : str | array of int
-        Order in which to plot data. 'type' groups by channel type, 'original'
-        plots in the order of ch_names, 'selection' uses Elekta's channel
-        groupings (only works for Neuromag data), 'position' groups the
-        channels by the positions of the sensors. 'selection' and 'position'
-        modes allow custom selections by using lasso selector on the topomap.
-        Pressing ``ctrl`` key while selecting allows appending to the current
-        selection. If array, only the channels in the array are plotted in the
-        given order. Defaults to 'type'.
+    order : array of int | None
+        Order in which to plot data. If the array is shorter than the number of
+        channels, only the given channels are plotted. If None (default), all
+        channels are plotted. If ``group_by`` is ``'position'`` or
+        ``'selection'``, the ``order`` parameter is used only for selecting the
+        channels to be plotted.
     show_options : bool
         If True, a dialog for options related to projection is shown.
     title : str | None
@@ -177,6 +176,32 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
         then excessive values are not shown, creating gaps in the traces.
     show_first_samp : bool
         If True, show time axis relative to the ``raw.first_samp``.
+    proj : bool
+        Whether to apply projectors prior to plotting (default is ``True``).
+        Individual projectors can be enabled/disabled interactively (see
+        Notes). This argument only affects the plot; use ``raw.apply_proj()``
+        to modify the data stored in the Raw object.
+    group_by : str
+        How to group channels. ``'type'`` groups by channel type,
+        ``'original'`` plots in the order of ch_names, ``'selection'`` uses
+        Elekta's channel groupings (only works for Neuromag data),
+        ``'position'`` groups the channels by the positions of the sensors.
+        ``'selection'`` and ``'position'`` modes allow custom selections by
+        using lasso selector on the topomap. Pressing ``ctrl`` key while
+        selecting allows appending to the current selection. Channels marked as
+        bad appear with red edges on the topomap. ``'type'`` and ``'original'``
+        groups the channels by type in butterfly mode whereas ``'selection'``
+        and ``'position'`` use regional grouping. ``'type'`` and ``'original'``
+        modes are overrided with ``order`` keyword.
+    butterfly : bool
+        Whether to start in butterfly mode. Defaults to False.
+    decim : int | 'auto'
+        Amount to decimate the data during display for speed purposes.
+        You should only decimate if the data are sufficiently low-passed,
+        otherwise aliasing can occur. The 'auto' mode (default) uses
+        the decimation that results in a sampling rate least three times
+        larger than ``min(info['lowpass'], lowpass)`` (e.g., a 40 Hz lowpass
+        will result in at least a 120 Hz displayed sample rate).
 
     Returns
     -------
@@ -193,6 +218,15 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
     can be to toggled with f11 key. To mark or un-mark a channel as bad, click
     on the rather flat segments of a channel's time series. The changes will be
     reflected immediately in the raw object's ``raw.info['bads']`` entry.
+
+    If projectors are present, a button labelled "Proj" in the lower right
+    corner of the plot window opens a secondary control window, which allows
+    enabling/disabling specific projectors individually. This provides a means
+    of interactively observing how each projector would affect the raw data if
+    it were applied.
+
+    Annotation mode is toggled by pressing 'a' and butterfly mode by pressing
+    'b'.
     """
     import matplotlib.pyplot as plt
     import matplotlib as mpl
@@ -229,7 +263,7 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
                         analog=False)
 
     # make a copy of info, remove projection (for now)
-    info = copy.deepcopy(raw.info)
+    info = raw.info.copy()
     projs = info['projs']
     info['projs'] = []
     n_times = raw.n_times
@@ -276,32 +310,39 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
         raise RuntimeError('Some channels not classified, please report '
                            'this problem')
 
-    # put them back to original or modified order for natral plotting
+    # put them back to original or modified order for natural plotting
     reord = np.argsort(inds)
     types = [types[ri] for ri in reord]
     if isinstance(order, string_types):
-        if order == 'original':
-            inds = inds[reord]
-        elif order in ['selection', 'position']:
-            selections, fig_selection = _setup_browser_selection(raw, order)
-        elif order != 'type':
-            raise ValueError('Unknown order type %s' % order)
+        group_by = order
+        warn('Using string order is deprecated and will not be allowed in '
+             '0.16. Use group_by instead.')
     elif isinstance(order, (np.ndarray, list)):
         # put back to original order first, then use new order
         inds = inds[reord][order]
+    elif order is not None:
+        raise ValueError('Unkown order type. Got %s.' % type(order))
+
+    if group_by in ['selection', 'position']:
+        selections, fig_selection = _setup_browser_selection(raw, group_by)
+        selections = {k: np.intersect1d(v, inds) for k, v in
+                      selections.items()}
+    elif group_by == 'original':
+        if order is None:
+            order = np.arange(len(inds))
+            inds = inds[reord[:len(order)]]
+    elif group_by != 'type':
+        raise ValueError('Unknown group_by type %s' % group_by)
 
     if not isinstance(event_color, dict):
         event_color = {-1: event_color}
-    else:
-        event_color = copy.deepcopy(event_color)  # we might modify it
+    event_color = dict((_ensure_int(key, 'event_color key'), event_color[key])
+                       for key in event_color)
     for key in event_color:
-        if not isinstance(key, int):
-            raise TypeError('event_color key "%s" was a %s not an int'
-                            % (key, type(key)))
         if key <= 0 and key != -1:
             raise KeyError('only key <= 0 allowed is -1 (cannot use %s)'
                            % key)
-
+    decim, data_picks = _handle_decim(info, decim, lowpass)
     # set up projection and data parameters
     duration = min(raw.times[-1], float(duration))
     first_time = raw._first_time if show_first_samp else 0
@@ -311,9 +352,11 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
                   n_channels=n_channels, scalings=scalings, types=types,
                   n_times=n_times, event_times=event_times, inds=inds,
                   event_nums=event_nums, clipping=clipping, fig_proj=None,
-                  first_time=first_time, added_label=list())
+                  first_time=first_time, added_label=list(), butterfly=False,
+                  group_by=group_by, orig_inds=inds.copy(), decim=decim,
+                  data_picks=data_picks)
 
-    if isinstance(order, string_types) and order in ['selection', 'position']:
+    if group_by in ['selection', 'position']:
         params['fig_selection'] = fig_selection
         params['selections'] = selections
         params['radio_clicked'] = partial(_radio_clicked, params=params)
@@ -343,6 +386,7 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
     if len(raw.info['projs']) > 0 and not raw.proj:
         ax_button = plt.subplot2grid((10, 10), (9, 9))
         params['ax_button'] = ax_button
+        params['apply_proj'] = proj
         opt_button = mpl.widgets.Button(ax_button, 'Proj')
         callback_option = partial(_toggle_options, params=params)
         opt_button.on_clicked(callback_option)
@@ -371,13 +415,13 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
     _layout_figure(params)
 
     # deal with projectors
-    if show_options is True:
+    if show_options:
         _toggle_options(None, params)
 
     callback_close = partial(_close_event, params=params)
     params['fig'].canvas.mpl_connect('close_event', callback_close)
     # initialize the first selection set
-    if isinstance(order, string_types) and order in ['selection', 'position']:
+    if group_by in ['selection', 'position']:
         _radio_clicked(fig_selection.radio.labels[0]._text, params)
         callback_selection_key = partial(_selection_key_press, params=params)
         callback_selection_scroll = partial(_selection_scroll, params=params)
@@ -387,6 +431,8 @@ def plot_raw(raw, events=None, duration=10.0, start=0.0, n_channels=20,
                                                    callback_selection_key)
         params['fig_selection'].canvas.mpl_connect('scroll_event',
                                                    callback_selection_scroll)
+    if butterfly:
+        _setup_butterfly(params)
 
     try:
         plt_show(show, block=block)
@@ -419,13 +465,16 @@ def _close_event(event, params):
     import matplotlib.pyplot as plt
     if 'fig_selection' in params:
         plt.close(params['fig_selection'])
-    if 'fig_annotation' is not None:
-        plt.close(params['fig_annotation'])
+    for fig in ['fig_annotation', 'fig_help', 'fig_proj']:
+        if params[fig] is not None:
+            plt.close(params[fig])
     plt.close(params['fig'])
 
 
 def _label_clicked(pos, params):
     """Select bad channels."""
+    if params['butterfly']:
+        return
     labels = params['ax'].yaxis.get_ticklabels()
     offsets = np.array(params['offsets']) + params['offsets'][0]
     line_idx = np.searchsorted(offsets, pos[1])
@@ -517,33 +566,72 @@ def _set_psd_plot_params(info, proj, picks, ax, area_mode):
             ax_list, make_label)
 
 
-def _convert_psds(psds, dB, scaling, unit, ch_names):
-    """Convert PSDs to dB (if necessary) and appropriate units."""
-    if dB:
-        where = np.where(psds.min(1) <= 0)[0]
-        if len(where) > 0:
-            raise ValueError("Infinite value in PSD for channel(s) %s. "
-                             "These channels might be dead." %
-                             ', '.join(ch_names[ii] for ii in where))
-        psds *= scaling * scaling
-        np.log10(psds, out=psds)
-        psds *= 10
-        ylabel = '%s/Hz (dB)' % unit
-    else:
+def _convert_psds(psds, dB, estimate, scaling, unit, ch_names):
+    """Convert PSDs to dB (if necessary) and appropriate units.
+
+    The following table summarizes the relationship between the value of
+    parameters ``dB`` and ``estimate``, and the type of plot and corresponding
+    units.
+
+    | dB    | estimate    | plot | units             |
+    |-------+-------------+------+-------------------|
+    | True  | 'power'     | PSD  | amp**2/Hz (dB)    |
+    | True  | 'amplitude' | ASD  | amp/sqrt(Hz) (dB) |
+    | True  | 'auto'      | PSD  | amp**2/Hz (dB)    |
+    | False | 'power'     | PSD  | amp**2/Hz         |
+    | False | 'amplitude' | ASD  | amp/sqrt(Hz)      |
+    | False | 'auto'      | ASD  | amp/sqrt(Hz)      |
+
+    where amp are the units corresponding to the variable, as specified by
+    ``unit``.
+    """
+    where = np.where(psds.min(1) <= 0)[0]
+    dead_ch = ', '.join(ch_names[ii] for ii in where)
+    if len(where) > 0:
+        if dB:
+            msg = "Infinite value in PSD for channel(s) %s. " \
+                  "These channels might be dead." % dead_ch
+        else:
+            msg = "Zero value in PSD for channel(s) %s. " \
+                  "These channels might be dead." % dead_ch
+        warn(msg)
+
+    if estimate == 'auto':
+        if dB:
+            estimate = 'power'
+        else:
+            estimate = 'amplitude'
+
+    if estimate == 'amplitude':
         np.sqrt(psds, out=psds)
         psds *= scaling
-        ylabel = '$\\frac{%s}{\\sqrt{Hz}}$' % unit
+        ylabel = r'$\mathrm{%s / \sqrt{Hz}}$' % unit
+    else:
+        psds *= scaling * scaling
+        ylabel = r'$\mathrm{%s^2}/Hz}$' % unit
+
+    if dB:
+        np.log10(np.maximum(psds, np.finfo(float).tiny), out=psds)
+        psds *= 10
+        ylabel += r'$\ \mathrm{(dB)}$'
+
     return ylabel
 
 
 @verbose
 def plot_raw_psd(raw, tmin=0., tmax=np.inf, fmin=0, fmax=np.inf, proj=False,
                  n_fft=None, picks=None, ax=None, color='black',
-                 area_mode='std', area_alpha=0.33, n_overlap=0, dB=True,
-                 average=None, show=True, n_jobs=1, line_alpha=None,
-                 spatial_colors=None, xscale='linear', verbose=None):
+                 area_mode='std', area_alpha=0.33, n_overlap=0,
+                 dB=True, estimate='auto', average=None, show=True, n_jobs=1,
+                 line_alpha=None, spatial_colors=None, xscale='linear',
+                 reject_by_annotation=True, verbose=None):
     """Plot the power spectral density across channels.
 
+    Different channel types are drawn in sub-plots. When the data has been
+    processed with a bandpass, lowpass or highpass filter, dashed lines
+    indicate the boundaries of the filter (--). The line noise frequency is
+    also indicated with a dashed line (-.).
+
     Parameters
     ----------
     raw : instance of io.Raw
@@ -582,7 +670,17 @@ def plot_raw_psd(raw, tmin=0., tmax=np.inf, fmin=0, fmax=np.inf, proj=False,
         The number of points of overlap between blocks. The default value
         is 0 (no overlap).
     dB : bool
-        If True, transform data to decibels. If False, plot amplitudes.
+        Plot Power Spectral Density (PSD), in units (amplitude**2/Hz (dB)) if
+        ``dB=True``, and ``estimate='power'`` or ``estimate='auto'``. Plot PSD
+        in units (amplitude**2/Hz) if ``dB=False`` and,
+        ``estimate='power'``. Plot Amplitude Spectral Density (ASD), in units
+        (amplitude/sqrt(Hz)), if ``dB=False`` and ``estimate='amplitude'`` or
+        ``estimate='auto'``. Plot ASD, in units (amplitude/sqrt(Hz) (db)), if
+        ``dB=True`` and ``estimate='amplitude'``.
+    estimate : str, {'auto', 'power', 'amplitude'}
+        Can be "power" for power spectral density (PSD), "amplitude" for
+        amplitude spectrum density (ASD), or "auto" (default), which uses
+        "power" when dB is True and "amplitude" otherwise.
     average : bool
         If False, the PSDs of all channels is displayed. No averaging
         is done and parameters area_mode and area_alpha are ignored. When
@@ -599,6 +697,13 @@ def plot_raw_psd(raw, tmin=0., tmax=np.inf, fmin=0, fmax=np.inf, proj=False,
         Whether to use spatial colors. Only used when ``average=False``.
     xscale : str
         Can be 'linear' (default) or 'log'.
+    reject_by_annotation : bool
+        Whether to omit bad segments from the data while computing the
+        PSD. If True, annotated segments with a description that starts
+        with 'bad' are omitted. Has no effect if ``inst`` is an Epochs or
+        Evoked object. Defaults to True.
+
+        .. versionadded:: 0.15.0
     verbose : bool, str, int, or None
         If not None, override default verbose level (see :func:`mne.verbose`
         and :ref:`Logging documentation <tut_logging>` for more).
@@ -628,6 +733,7 @@ def plot_raw_psd(raw, tmin=0., tmax=np.inf, fmin=0, fmax=np.inf, proj=False,
     line_alpha = float(line_alpha)
 
     psd_list = list()
+    ylabels = list()
     if n_fft is None:
         tmax = raw.times[-1] if not np.isfinite(tmax) else tmax
         n_fft = min(np.diff(raw.time_as_index([tmin, tmax]))[0] + 1, 2048)
@@ -635,9 +741,11 @@ def plot_raw_psd(raw, tmin=0., tmax=np.inf, fmin=0, fmax=np.inf, proj=False,
         ax = ax_list[ii]
         psds, freqs = psd_welch(raw, tmin=tmin, tmax=tmax, picks=picks,
                                 fmin=fmin, fmax=fmax, proj=proj, n_fft=n_fft,
-                                n_overlap=n_overlap, n_jobs=n_jobs)
+                                n_overlap=n_overlap, n_jobs=n_jobs,
+                                reject_by_annotation=reject_by_annotation)
 
-        ylabel = _convert_psds(psds, dB, scalings_list[ii], units_list[ii],
+        ylabel = _convert_psds(psds, dB, estimate, scalings_list[ii],
+                               units_list[ii],
                                [raw.ch_names[pi] for pi in picks])
 
         if average:
@@ -660,10 +768,13 @@ def plot_raw_psd(raw, tmin=0., tmax=np.inf, fmin=0, fmax=np.inf, proj=False,
 
         if make_label:
             if ii == len(picks_list) - 1:
-                ax.set_xlabel('Freq (Hz)')
+                ax.set_xlabel('Frequency (Hz)')
             ax.set_ylabel(ylabel)
             ax.set_title(titles_list[ii])
             ax.set_xlim(freqs[0], freqs[-1])
+
+        ylabels.append(ylabel)
+
     for key, ls in zip(['lowpass', 'highpass', 'line_freq'],
                        ['--', '--', '-.']):
         if raw.info[key] is not None:
@@ -687,8 +798,8 @@ def plot_raw_psd(raw, tmin=0., tmax=np.inf, fmin=0, fmax=np.inf, proj=False,
         for this_type in valid_channel_types:
             if this_type in types:
                 ch_types_used.append(this_type)
-        unit = 'dB/Hz' if dB else '$1/\\sqrt{Hz}$)'
-        units = {t: 'PSD (%s)' % unit for t in ch_types_used}
+        unit = ''
+        units = {t: yl for t, yl in zip(ch_types_used, ylabels)}
         titles = {c: t for c, t in zip(ch_types_used, titles_list)}
         picks = np.arange(len(psd_list))
         if not spatial_colors:
@@ -720,11 +831,11 @@ def _prepare_mne_browse_raw(params, title, bgcolor, color, bad_color, inds,
     if size is not None:
         size = size.split(',')
         size = tuple([float(s) for s in size])
+        size = tuple([float(s) for s in size])
 
     fig = figure_nobar(facecolor=bgcolor, figsize=size)
-    fig.canvas.set_window_title('mne_browse_raw')
+    fig.canvas.set_window_title(title if title else "Raw")
     ax = plt.subplot2grid((10, 10), (0, 1), colspan=8, rowspan=9)
-    ax.set_title(title, fontsize=12)
     ax_hscroll = plt.subplot2grid((10, 10), (9, 1), colspan=8)
     ax_hscroll.get_yaxis().set_visible(False)
     ax_hscroll.set_xlabel('Time (s)')
@@ -790,7 +901,7 @@ def _prepare_mne_browse_raw(params, title, bgcolor, color, bad_color, inds,
 
     vertline_color = (0., 0.75, 0.)
     params['ax_vertline'] = ax.plot([0, 0], ax.get_ylim(),
-                                    color=vertline_color, zorder=-1)[0]
+                                    color=vertline_color, zorder=4)[0]
     params['ax_vertline'].ch_name = ''
     params['vertline_t'] = ax_hscroll.text(params['first_time'], 1, '',
                                            color=vertline_color,
@@ -803,12 +914,16 @@ def _prepare_mne_browse_raw(params, title, bgcolor, color, bad_color, inds,
     ax.set_xlim(params['t_start'], params['t_start'] + params['duration'],
                 False)
 
-    params['lines'] = [ax.plot([np.nan], antialiased=False, linewidth=0.5)[0]
+    params['lines'] = [ax.plot([np.nan], antialiased=True, linewidth=0.5)[0]
                        for _ in range(n_ch)]
     ax.set_yticklabels(['X' * max([len(ch) for ch in info['ch_names']])])
     params['fig_annotation'] = None
+    params['fig_help'] = None
     params['segment_line'] = None
 
+    # default key to close window
+    params['close_key'] = 'escape'
+
 
 def _plot_raw_traces(params, color, bad_color, event_lines=None,
                      event_color=None):
@@ -816,13 +931,21 @@ def _plot_raw_traces(params, color, bad_color, event_lines=None,
     lines = params['lines']
     info = params['info']
     inds = params['inds']
-    n_channels = params['n_channels']
+    butterfly = params['butterfly']
+    if butterfly:
+        n_channels = len(params['offsets'])
+        ch_start = 0
+        offsets = params['offsets'][inds]
+    else:
+        n_channels = params['n_channels']
+        ch_start = params['ch_start']
+        offsets = params['offsets']
     params['bad_color'] = bad_color
     labels = params['ax'].yaxis.get_ticklabels()
     # do the plotting
     tick_list = list()
     for ii in range(n_channels):
-        ch_ind = ii + params['ch_start']
+        ch_ind = ii + ch_start
         # let's be generous here and allow users to pass
         # n_channels per view >= the number of traces available
         if ii >= len(lines):
@@ -831,26 +954,39 @@ def _plot_raw_traces(params, color, bad_color, event_lines=None,
             # scale to fit
             ch_name = info['ch_names'][inds[ch_ind]]
             tick_list += [ch_name]
-            offset = params['offsets'][ii]
-
+            offset = offsets[ii]
             # do NOT operate in-place lest this get screwed up
             this_data = params['data'][inds[ch_ind]] * params['scale_factor']
             this_color = bad_color if ch_name in info['bads'] else color
-            this_z = 0 if ch_name in info['bads'] else 1
+
             if isinstance(this_color, dict):
                 this_color = this_color[params['types'][inds[ch_ind]]]
 
-            # subtraction here gets corect orientation for flipped ylim
-            lines[ii].set_ydata(offset - this_data)
-            lines[ii].set_xdata(params['times'] + params['first_time'])
+            if inds[ch_ind] in params['data_picks']:
+                this_decim = params['decim']
+            else:
+                this_decim = 1
+            this_t = params['times'][::this_decim] + params['first_time']
+            # subtraction here gets correct orientation for flipped ylim
+            lines[ii].set_ydata(offset - this_data[..., ::this_decim])
+            lines[ii].set_xdata(this_t)
             lines[ii].set_color(this_color)
-            lines[ii].set_zorder(this_z)
             vars(lines[ii])['ch_name'] = ch_name
             vars(lines[ii])['def_color'] = color[params['types'][inds[ch_ind]]]
-
-            # set label color
-            this_color = bad_color if ch_name in info['bads'] else 'black'
-            labels[ii].set_color(this_color)
+            this_z = 0 if ch_name in info['bads'] else 1
+            if butterfly:
+                if ch_name not in info['bads']:
+                    if params['types'][ii] == 'mag':
+                        this_z = 2
+                    elif params['types'][ii] == 'grad':
+                        this_z = 3
+                for label in labels:
+                    label.set_color('black')
+            else:
+                # set label color
+                this_color = bad_color if ch_name in info['bads'] else 'black'
+                labels[ii].set_color(this_color)
+            lines[ii].set_zorder(this_z)
         else:
             # "remove" lines
             lines[ii].set_xdata([])
@@ -884,6 +1020,12 @@ def _plot_raw_traces(params, color, bad_color, event_lines=None,
             else:
                 line.set_xdata([])
                 line.set_ydata([])
+        if len(event_times) <= 50:
+            params['ax'].texts = []
+            for ev_time, ev_num in zip(event_times, event_nums):
+                if -1 in event_color or ev_num in event_color:
+                    params['ax'].text(ev_time, -0.05, ev_num, fontsize=8,
+                                      ha='center')
 
     if 'segments' in params:
         while len(params['ax'].collections) > 0:
@@ -909,7 +1051,8 @@ def _plot_raw_traces(params, color, bad_color, event_lines=None,
     params['ax'].set_xlim(params['times'][0] + params['first_time'],
                           params['times'][0] + params['first_time'] +
                           params['duration'], False)
-    params['ax'].set_yticklabels(tick_list)
+    if not butterfly:
+        params['ax'].set_yticklabels(tick_list, rotation=0)
     if 'fig_selection' not in params:
         params['vsel_patch'].set_y(params['ch_start'])
     params['fig'].canvas.draw()
@@ -923,7 +1066,8 @@ def _plot_raw_traces(params, color, bad_color, event_lines=None,
 def plot_raw_psd_topo(raw, tmin=0., tmax=None, fmin=0., fmax=100., proj=False,
                       n_fft=2048, n_overlap=0, layout=None, color='w',
                       fig_facecolor='k', axis_facecolor='k', dB=True,
-                      show=True, block=False, n_jobs=1, verbose=None):
+                      show=True, block=False, n_jobs=1, axes=None,
+                      verbose=None):
     """Plot channel-wise frequency spectra as topography.
 
     Parameters
@@ -966,6 +1110,8 @@ def plot_raw_psd_topo(raw, tmin=0., tmax=None, fmin=0., fmax=100., proj=False,
         May not work on all systems / platforms. Defaults to False.
     n_jobs : int
         Number of jobs to run in parallel. Defaults to 1.
+    axes : instance of matplotlib Axes | None
+        Axes to plot into. If None, axes will be created.
     verbose : bool, str, int, or None
         If not None, override default verbose level (see :func:`mne.verbose`
         and :ref:`Logging documentation <tut_logging>` for more).
@@ -998,7 +1144,7 @@ def plot_raw_psd_topo(raw, tmin=0., tmax=None, fmin=0., fmax=100., proj=False,
                      click_func=click_func, layout=layout,
                      axis_facecolor=axis_facecolor,
                      fig_facecolor=fig_facecolor, x_label='Frequency (Hz)',
-                     unified=True, y_label=y_label)
+                     unified=True, y_label=y_label, axes=axes)
 
     try:
         plt_show(show, block=block)
@@ -1019,16 +1165,17 @@ def _set_custom_selection(params):
     _set_radio_button(labels.index('Custom'), params=params)
 
 
-def _setup_browser_selection(raw, kind):
+def _setup_browser_selection(raw, kind, selector=True):
     """Organize browser selections."""
     import matplotlib.pyplot as plt
     from matplotlib.widgets import RadioButtons
     from ..selection import (read_selection, _SELECTIONS, _EEG_SELECTIONS,
                              _divide_to_regions)
     from ..utils import _get_stim_channel
-    if kind in ('position'):
+    if kind == 'position':
         order = _divide_to_regions(raw.info)
         keys = _SELECTIONS[1:]  # no 'Vertex'
+        kind = 'position'
     elif 'selection':
         from ..io import RawFIF, RawArray
         if not isinstance(raw, (RawFIF, RawArray)):
@@ -1055,6 +1202,8 @@ def _setup_browser_selection(raw, kind):
     if len(misc) > 0:
         order['Misc'] = misc
     keys = np.concatenate([keys, ['Misc']])
+    if not selector:
+        return order
     fig_selection = figure_nobar(figsize=(2, 6), dpi=80)
     fig_selection.canvas.set_window_title('Selection')
     rax = plt.subplot2grid((6, 1), (2, 0), rowspan=4, colspan=1)
